#  --------------------------------------------------------------------------
#  file: "_patches/sketchup_rb/sketchup_rb_fix_inputbox.rb"
#{  --------------------------------------------------------------------------
#  Copyright 2005-2012, Google, Inc.
#  --------------------------------------------------------------------------
#
#  The "Tools/sketchup.rb" file defines a number of useful utilities
#     that are used by other Ruby scripts.
#
#  THIS file "_patches/sketchup_rb/sketchup_rb_fix_inputbox.rb", fixes certain
#    issues in the "Tools/sketchup.rb" file by overriding methods, etc.
#
#  --------------------------------------------------------------------------
#  Permission to use, copy, modify, and distribute this software for 
#  any purpose and without fee is hereby granted, provided that the above
#  copyright notice appear in all copies.
#  --------------------------------------------------------------------------
#  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#  --------------------------------------------------------------------------
#  Versions:
#
#   1.0.0  :  2012-02-29    Dan Rathbun - Palm Bay, FL, USA
#   |
#   + Add a inputbox_with_error_trap() function to module UI.
#   ~ Alias global inputbox() to UI.inputbox_with_error_trap()
#}
#  --------------------------------------------------------------------------

require('sketchup.rb')

#  ===========================================================================

  # BACKWARD COMPATIBILITY

  def inputbox(*args)
      UI.inputbox_with_error_trap(*args)
  end

#  ===========================================================================

  module UI

    class << self  #{ PROXY CLASS
    
      public

      unless method_defined?(:inputbox_with_error_trap)
      #  ---------------------------------------------------------------------------
      #{ inputbox_with_error_trap(*args)
      #  ---------------------------------------------------------------------------
      #{
      #  This is a wrapper method for UI.inputbox, which will raise an
      #  exception if it can't convert the string entered for one of
      #  the values into the right type.
      #
      #  This method will trap the exception and display an error
      #  dialog and then prompt for the values again.
      #
      #  Args:   Exactly the same as UI.inputbox.
      #  ---------------------------------------------------------------------------
      #
      #  This patch does more argument list validation, and issues more
      #    informative exception messages, than UI.inputbox() does.
      #    The inputbox() wrapper did no argument list validation at
      #    all; and coding errors would result in a messagebox loop,
      #    that required killing Sketchup with the Task Manager.
      #
      #}
      def inputbox_with_error_trap(*args)
          #puts "UI.inputbox_with_error_trap() called." ##### debug
          #
          #  We must trap any ArgumentErrors that might be produced by
          #    errors in the actual argumentlist to UI.inputbox, HERE,
          #    because errors produced later, by invalid user input
          #    also raise an ArgumentError exception. If we don't trap
          #    now, we won't be able to tell the difference later.
          #
          msg = 'method UI::inputbox(): '
          if args.empty?
              # Would get caught in a "too few arguments" loop,
              # when inputbox was called with no arguments.
              # This forced killing Sketchup with the TaskManager.
              msg << 'wrong number of arguments (0 for minimum of 1).'
              raise(ArgumentError,msg,caller)
          end
          # Arg 0 : Array of Strings, for Input Field Labels.
          #         Can also contain objects that respond to a to_s() method call.
          if( not args[0].is_a?(Array) )
              msg << "wrong argument type #{args[0].class.name} (for arg 0,) expected Array."
              raise(TypeError,msg,caller)
          else
              faulty = -1
              args[0].each_with_index{|e,i| break (faulty=i) unless e.respond_to?(:to_s) || e.is_a?(String) }
              if faulty>=0
                  msg << "wrong type for Input Field Label, element (#{faulty}) in Array argument (0). "
                  msg << "Element objects must evaluate to type String, or respond to 'to_s' method."
                  raise(TypeError,msg,caller)
              end
          end
          # Arg 1 : Array of Values, for Input Field Defaults.
          if args.size>1
              if args[1].nil?
                  args[1] = [] # be nice and stuff an empty array in the arg slot.
              elsif( not args[1].is_a?(Array) )
                  msg << "wrong argument type #{args[1].class.name} (for arg 1,) expected Array."
                  raise(TypeError,msg,caller)
              end
          end
          # Arg 2 : Variant:
          if args.size>=4
            # Arg 2 : Array of options Strings for Input Fields.
            #   Each String, a pipe-separated list of Values.
            unless args[2].is_a?(Array)
                msg << "wrong argument type #{args[2].class.name} (for arg 2,) expected Array."
                raise(TypeError,msg,caller)
            else
              faulty = -1
              args[2].each_with_index{|e,i| break (faulty=i) unless e.respond_to?(:to_s) || e.is_a?(String) }
              if faulty>=0
                  msg << "wrong type for Input Field Options, element (#{faulty}) in Array argument (2). "
                  msg << "Element objects must evaluate to type String. Options within String must be separated by a pipe character, '|'."
                  raise(TypeError,msg,caller)
              end
            end
            # Arg 3 : String title of Inputbox.
            unless args[3].is_a?(String)
                msg << "wrong argument type #{args[3].class.name} (for arg 3,) expected String."
                raise(TypeError,msg,caller)
            end
          elsif args.size==3 # only 3 arguments:
          # Arg 2 : String title of Inputbox.
            unless args[2].is_a?(String)
                msg << "wrong argument type #{args[2].class.name} (for arg 2,) expected String."
                raise(TypeError,msg,caller)
            end
          end
          # if eval gets here the argument list is OK.
          results = nil
          begin
              results = UI.inputbox(*args)
          rescue ArgumentError => e
              #
              msg = e.message.strip << "      \n\n"
              msg[0,1]= msg[0,1].upcase
              #
              UI.messagebox( "\n" << msg )
              #
              retry
              #
          rescue Exception
              raise()
          end
          results
      end #}
      #
      end # method definition check

    end #} PROXY CLASS

  end # module UI

#  ===========================================================================


  #{# RUN ONCE
  #
  unless file_loaded?(File.basename(__FILE__))

    file_loaded(File.basename(__FILE__))

  end
  #
  #}#
