#  --------------------------------------------------------------------------
#  file: "_patches/sketchup_rb/sketchup_rb_fix_require_all.rb"
#{  --------------------------------------------------------------------------
#  Copyright 2005-2012, Google, Inc.
#  --------------------------------------------------------------------------
#
#  The "Tools/sketchup.rb" file defines a number of useful utilities
#     that are used by other Ruby scripts.
#
#  THIS file "_patches/sketchup_rb/sketchup_rb_fix_require_all.rb",
#    fixes certain issues in the "Tools/sketchup.rb" file by overriding
#    methods, etc.
#
#  --------------------------------------------------------------------------
#  Permission to use, copy, modify, and distribute this software for 
#  any purpose and without fee is hereby granted, provided that the above
#  copyright notice appear in all copies.
#  --------------------------------------------------------------------------
#  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#  --------------------------------------------------------------------------
#  Versions:
#
#   1.0.0  :  2012-03-02    Dan Rathbun - Palm Bay, FL, USA
#   |
#   ~ Override global require_all() method to fix errors and improve it.
#}
#  --------------------------------------------------------------------------

require('sketchup.rb')

#  ===========================================================================


  #  ---------------------------------------------------------------------------
  #{ require_all( dirname )
  #  ---------------------------------------------------------------------------
  #{
  #  By default, SketchUp automatically loads (using require) all files with
  #  the .rb extension in the plugins directory.  This function can be used
  #  to automatically load all .rb files from a different directory also.  to
  #  use this add a call like the following to a file in the plugins directory
  #  require_all "MyRubyScripts"
  #  ---------------------------------------------------------------------------
  #
  #  This patch implements several error handling features:
  #
  #    (1) The dirname is checked for existence.
  #          If not, a Invalid Directory exception is raised.
  #
  #    (2) The dirname (if exists,) is checked for {rb,rbs} file emptiness.
  #          If empty, the path is pushed into $LOAD_PATH, and nil is returned.
  #
  #    (3) The begin .. rescue .. else block is moved INSIDE the rbfiles each
  #          block, so as to individually trap load errors for each file.
  #          Any such errors, are logged to an internal 'errors' array.
  #          Any successful loads are logged to a similar 'loaded' array.
  #          Each element in these arrays is a hash. (see below)
  #        This allows the each block to continue processing the remaining
  #          files in the directory, instead of being stopped by the first
  #          load error it encounters.
  #
  #    (4) The old edition, did not explictily return a useful value.
  #          This override returns an array, such that:
  #
  #          retval, errors, loaded = require_all( dirname )
  #          
  #          retval  =  nil, if dirname had no {rb,rbs} files to load.
  #                     (errors & loaded will also be nil, in this case.)
  #                  =  true if no errors occurred, in loading any file(s).
  #                  =  false if any errors occurred at all.
  #          
  #          errors  = nil, if it's internal array was empty, ie, no errors.
  #                  = Array of Hash(es) for each file error:
  #                    { :file => filename,
  #                      :exception => exception,
  #                      :path => path
  #                    }
  #                    You can then get the exception class, message and
  #                    backtrace, by querying the hash, example: 
  #                    puts( errors[3][:exception].backtrace )
  #          
  #          loaded  = nil, if it's internal array was empty, ie, no loads.
  #                  = Array of Hash(es) for each file loaded:
  #                    { :file => filename,
  #                      :success => true|false,
  #                      :preloaded => true|false
  #                    }
  #                    :success is true if THIS call loaded the file,
  #                             or false if it had been preloaded.
  #                    :preloaded has an inverse value to :success.
  #}
  def require_all(dirname)
      #
      dirname = File.expand_path(dirname) if dirname[0,1]=='~'
      #
      unless Kernel.test( ?d, dirname ) # exists and is a directory?
          stack =( caller(0).size < 2 ? caller(0) : caller(1) )
          raise(Errno::ENOENT,": require_all(): Invalid directory: '#{dirname}'.",stack)
      end
      #
      prevDir = Dir.getwd()
      Dir.chdir(dirname)
      rbfiles = Dir["*.{rb,rbs,dll,so,o}"] # dylib, bundle ?? for MAC
      Dir.chdir(prevDir)
      #
      if rbfiles.empty?
          puts( "Notice: require_all(): No files to load from '#{dirname}'\n" ) unless $VERBOSE.nil?
          # append dirname onto array $LOAD_PATH
          $:.push( dirname )
          return nil
      else
          # temporarily insert dirname into first position of $LOAD_PATH
          ass = $:.size # save the position where dirname will be
          $:.unshift( dirname )
          errors = []
          loaded = []
          retval = true
          rbfiles.each {|f|
              success = false
              # Both Sketchup::require() and the global require() methods
              #   return false if the script file was already loaded.
              begin
                  success =( File.extname(f)=='.rbs' ? Sketchup::require( f ) : require( f ) )
              rescue Exception => e
                  retval = false
                  errors << {:file => f, :exception => e, :path => dirname }
                  # now the loop can continue processing the rbfiles list ...
              else
                  # Sketchup::require() returns 0, and global require() returns true,
                  #    if the script file was first successfully loaded by this call.
                  success =(success ? true : false)
                  loaded << {:file => f, :success => success, :preloaded => (not success) }
                  file_loaded(File.basename(f)) unless file_loaded?(File.basename(f))
              end
          }
          # move dirname to ass position in array $LOAD_PATH
          $:.delete( dirname )
          $:.insert( ass, dirname ) # perhaps no longer the very last position
          $:.uniq!
          #
          return retval, ( errors.empty? ? nil : errors ), ( loaded.empty? ? nil : loaded )
          #
      end
      #
  end #}

#  ===========================================================================


  #{# RUN ONCE
  #
  unless file_loaded?(File.basename(__FILE__))

    file_loaded(File.basename(__FILE__))

  end
  #
  #}#
